#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import subprocess
import sys
import pip


def _get_pypackages_lib_path(script_path=None):
    """returns path in compliance with PEP 582
    https://www.python.org/dev/peps/pep-0582/
    """
    if script_path:
        # use __pypackages__ relative to the script being run
        pypackages = os.path.join(os.path.dirname(script_path), "__pypackages__")
    else:
        pypackages = "__pypackages__"

    return os.path.join(
        pypackages,
        str(sys.version_info.major) + "." + str(sys.version_info.minor),
        "lib",
    )


def _get_env(script_path=None):
    env = dict(os.environ)
    env["PYTHONPATH"] = os.path.pathsep.join(
        [".", _get_pypackages_lib_path(script_path)]
        + os.getenv("PYTHONPATH", "").split(os.path.pathsep)
    )
    return env


def _get_script_path():
    for arg in sys.argv[1:]:
        if not arg.startswith("-"):
            return os.path.abspath(arg)
    return None


def _get_pip_install_args(pip_args):
    if "install" in pip_args:
        target = [
            "--target",
            _get_pypackages_lib_path(),
        ]

        if (
            pip.__version__.startswith("9.") or pip.__version__.startswith("10.")
        ) and "--system" not in pip_args:
            target += ["--system"]
    else:
        target = []
    return target


def pythonloc():
    args = [sys.executable] + sys.argv[1:]
    script_path = _get_script_path()
    os.execve(sys.executable, args, _get_env(script_path))


def piploc():
    pip_args = sys.argv[1:]
    install_args = _get_pip_install_args(pip_args)
    args = [sys.executable] + ["-m", "pip"] + pip_args + install_args
    os.execve(sys.executable, args, _get_env())


def pipfreezeloc():
    cmd = [sys.executable, "-m", "pip", "freeze"]
    p = subprocess.Popen(cmd, env=_get_env(), stdout=subprocess.PIPE)
    try:
        outs, errs = p.communicate()
        if outs is None:
            all_reqs = set()
        else:
            all_reqs = set(outs.decode().split("\n"))
    except Exception:
        p.kill()
        exit("failed to run pip freeze")

    p = subprocess.Popen(cmd, stdout=subprocess.PIPE)
    try:
        outs, errs = p.communicate()
        if outs is None:
            sys_reqs = set()
        else:
            sys_reqs = set(outs.decode().split("\n"))
    except Exception:
        p.kill()
        exit("failed to run pip freeze")
    for i in sorted(all_reqs - sys_reqs):
        print(i)


if __name__ == "__main__":
    pythonloc()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# setuptools is a fully-featured, actively-maintained, and stable
# library designed to facilitate packaging Python projects.
# setup.py is the build script for setuptools.
# It tells setuptools about your package (such as the name and version)
# as well as which code files to include.
# https://packaging.python.org/tutorials/packaging-projects/
# https://setuptools.readthedocs.io/en/latest/

import io
import os
from setuptools import find_packages, setup

CURDIR = os.path.abspath(os.path.dirname(__file__))

with io.open(os.path.join(CURDIR, "README.md"), "r", encoding="utf-8") as f:
    README = f.read()

setup(
    name="pythonloc",
    version="0.1.2.2",
    author="Chad Smith",
    author_email="chadsmith.software@gmail.com",
    description="Run Python using packages from local directory __pypackages__",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/cs01/pythonloc",
    packages=find_packages(exclude=["contrib", "docs", "tests*"]),
    include_package_data=True,
    keywords=[],
    scripts=[],
    entry_points={
        "console_scripts": [
            "pythonloc = pythonloc.pythonloc:pythonloc",
            "piploc = pythonloc.pythonloc:piploc",
            "pipfreezeloc = pythonloc.pythonloc:pipfreezeloc",
        ]
    },
    zip_safe=False,
    install_requires=['pipx>=0.12.2.0;python_version>="3.6"'],
    python_requires=">=2.7",
    # license and classifier list:
    # https://pypi.org/pypi?%3Aaction=list_classifiers
    license="License :: OSI Approved :: MIT License",
    classifiers=[
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
)
